@@ -126,7 +126,8 @@ typedef enum {
   GRN_PLUGIN_ERROR = -75,
   GRN_SCORER_ERROR = -76,
   GRN_CANCEL = -77,
-  GRN_WINDOW_FUNCTION_ERROR = -78
+  GRN_WINDOW_FUNCTION_ERROR = -78,
+  GRN_ZSTD_ERROR = -79
 } grn_rc;
 
 GRN_API grn_rc grn_init(void);
@@ -247,6 +248,10 @@ GRN_API grn_ctx *grn_ctx_open(int flags);
 GRN_API grn_rc grn_ctx_close(grn_ctx *ctx);
 GRN_API grn_rc grn_ctx_set_finalizer(grn_ctx *ctx, grn_proc_func *func);
 
+GRN_API grn_rc grn_ctx_push_temporary_open_space(grn_ctx *ctx);
+GRN_API grn_rc grn_ctx_pop_temporary_open_space(grn_ctx *ctx);
+GRN_API grn_rc grn_ctx_merge_temporary_open_space(grn_ctx *ctx);
+
 GRN_API grn_encoding grn_get_default_encoding(void);
 GRN_API grn_rc grn_set_default_encoding(grn_encoding encoding);
 
@@ -327,6 +332,7 @@ typedef uint32_t grn_column_flags;
 #define GRN_OBJ_COMPRESS_LZ4           (0x02<<4)
 /* Just for backward compatibility. We'll remove it at 5.0.0. */
 #define GRN_OBJ_COMPRESS_LZO           GRN_OBJ_COMPRESS_LZ4
+#define GRN_OBJ_COMPRESS_ZSTD          (0x03<<4)
 
 #define GRN_OBJ_WITH_SECTION           (0x01<<7)
 #define GRN_OBJ_WITH_WEIGHT            (0x01<<8)
@@ -701,7 +707,8 @@ typedef enum {
 /* Just for backward compatibility. We'll remove it at 5.0.0. */
 #define GRN_INFO_SUPPORT_LZO GRN_INFO_SUPPORT_LZ4
   GRN_INFO_NORMALIZER,
-  GRN_INFO_TOKEN_FILTERS
+  GRN_INFO_TOKEN_FILTERS,
+  GRN_INFO_SUPPORT_ZSTD
 } grn_info_type;
 
 GRN_API grn_obj *grn_obj_get_info(grn_ctx *ctx, grn_obj *obj, grn_info_type type, grn_obj *valuebuf);
@@ -1016,6 +1023,7 @@ GRN_API grn_rc grn_snip_get_result(grn_ctx *ctx, grn_obj *snip, const unsigned i
 #define GRN_LOG_TITLE                  (0x01<<1)
 #define GRN_LOG_MESSAGE                (0x01<<2)
 #define GRN_LOG_LOCATION               (0x01<<3)
+#define GRN_LOG_PID                    (0x01<<4)
 
 /* Deprecated since 2.1.2. Use grn_logger instead. */
 typedef struct _grn_logger_info grn_logger_info;
@@ -1056,6 +1064,24 @@ GRN_API grn_log_level grn_logger_get_max_level(grn_ctx *ctx);
 # define GRN_ATTRIBUTE_PRINTF(fmt_pos)
 #endif /* __GNUC__ */
 
+#if defined(__clang__)
+# if __has_attribute(__alloc_size__)
+#  define HAVE_ALLOC_SIZE_ATTRIBUTE
+# endif /* __has_attribute(__alloc_size__) */
+#elif defined(__GNUC__)
+# define HAVE_ALLOC_SIZE_ATTRIBUTE
+#endif /* __clang__ */
+
+#ifdef HAVE_ALLOC_SIZE_ATTRIBUTE
+# define GRN_ATTRIBUTE_ALLOC_SIZE(size) \
+  __attribute__ ((alloc_size(size)))
+# define GRN_ATTRIBUTE_ALLOC_SIZE_N(n, size) \
+  __attribute__ ((alloc_size(n, size)))
+#else
+# define GRN_ATTRIBUTE_ALLOC_SIZE(size)
+# define GRN_ATTRIBUTE_ALLOC_SIZE_N(n, size)
+#endif /* HAVE_ALLOC_SIZE_ATTRIBUTE */
+
 GRN_API void grn_logger_put(grn_ctx *ctx, grn_log_level level,
                             const char *file, int line, const char *func, const char *fmt, ...) GRN_ATTRIBUTE_PRINTF(6);
 GRN_API void grn_logger_putv(grn_ctx *ctx,
@@ -1075,6 +1101,8 @@ GRN_API grn_bool grn_logger_pass(grn_ctx *ctx, grn_log_level level);
 
 GRN_API void grn_default_logger_set_max_level(grn_log_level level);
 GRN_API grn_log_level grn_default_logger_get_max_level(void);
+GRN_API void grn_default_logger_set_flags(int flags);
+GRN_API int grn_default_logger_get_flags(void);
 GRN_API void grn_default_logger_set_path(const char *path);
 GRN_API const char *grn_default_logger_get_path(void);
 GRN_API void grn_default_logger_set_rotate_threshold_size(off_t threshold);
