@@ -155,13 +155,19 @@ default_logger_log(grn_ctx *ctx, grn_log_level level,
       }
     }
     if (default_logger_file) {
+      char label = *(slev + level);
       int written;
       if (location && *location) {
-        written = fprintf(default_logger_file, "%s|%c|%s %s %s\n",
-                          timestamp, *(slev + level), title, message, location);
+        if (title && *title) {
+          written = fprintf(default_logger_file, "%s|%c|%s: %s %s\n",
+                            timestamp, label, location, title, message);
+        } else {
+          written = fprintf(default_logger_file, "%s|%c|%s: %s\n",
+                            timestamp, label, location, message);
+        }
       } else {
-        written = fprintf(default_logger_file, "%s|%c|%s %s\n", timestamp,
-                          *(slev + level), title, message);
+        written = fprintf(default_logger_file, "%s|%c|%s %s\n",
+                          timestamp, label, title, message);
       }
       if (written > 0) {
         default_logger_size += written;
@@ -236,6 +242,21 @@ grn_default_logger_get_max_level(void)
   return default_logger.max_level;
 }
 
+void
+grn_default_logger_set_flags(int flags)
+{
+  default_logger.flags = flags;
+  if (current_logger.log == default_logger_log) {
+    current_logger.flags = flags;
+  }
+}
+
+int
+grn_default_logger_get_flags(void)
+{
+  return default_logger.flags;
+}
+
 void
 grn_default_logger_set_path(const char *path)
 {
@@ -394,6 +415,9 @@ grn_logger_putv(grn_ctx *ctx,
     if (current_logger.flags & GRN_LOG_LOCATION) {
       grn_snprintf(lbuf, LBUFSIZE, LBUFSIZE,
                    "%d %s:%d %s()", getpid(), file, line, func);
+    } else if (current_logger.flags & GRN_LOG_PID) {
+      grn_snprintf(lbuf, LBUFSIZE, LBUFSIZE,
+                   "%d", getpid());
     } else {
       lbuf[0] = '\0';
     }
