@@ -271,6 +271,9 @@ grn_proc_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   case GRN_PROC_SCORER :
     GRN_TEXT_PUTS(ctx, buf, "scorer");
     break;
+  case GRN_PROC_WINDOW_FUNCTION :
+    GRN_TEXT_PUTS(ctx, buf, "window-function");
+    break;
   }
   GRN_TEXT_PUTS(ctx, buf, " ");
 
@@ -450,7 +453,117 @@ grn_vector_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *vector)
 static grn_rc
 grn_accessor_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 {
-  return grn_column_name_(ctx, obj, buf);
+  grn_accessor *accessor = (grn_accessor *)obj;
+
+  GRN_TEXT_PUTS(ctx, buf, "#<accessor ");
+  for (; accessor; accessor = accessor->next) {
+    grn_bool show_obj_name = GRN_FALSE;
+    grn_bool show_obj_domain_name = GRN_FALSE;
+
+    if (accessor != (grn_accessor *)obj) {
+      GRN_TEXT_PUTS(ctx, buf, ".");
+    }
+    switch (accessor->action) {
+    case GRN_ACCESSOR_GET_ID :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_ID,
+                   GRN_COLUMN_NAME_ID_LEN);
+      show_obj_name = GRN_TRUE;
+      break;
+    case GRN_ACCESSOR_GET_KEY :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_KEY,
+                   GRN_COLUMN_NAME_KEY_LEN);
+      show_obj_name = GRN_TRUE;
+      break;
+    case GRN_ACCESSOR_GET_VALUE :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_VALUE,
+                   GRN_COLUMN_NAME_VALUE_LEN);
+      show_obj_name = GRN_TRUE;
+      break;
+    case GRN_ACCESSOR_GET_SCORE :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_SCORE,
+                   GRN_COLUMN_NAME_SCORE_LEN);
+      break;
+    case GRN_ACCESSOR_GET_NSUBRECS :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_NSUBRECS,
+                   GRN_COLUMN_NAME_NSUBRECS_LEN);
+      break;
+    case GRN_ACCESSOR_GET_MAX :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_MAX,
+                   GRN_COLUMN_NAME_MAX_LEN);
+      break;
+    case GRN_ACCESSOR_GET_MIN :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_MIN,
+                   GRN_COLUMN_NAME_MIN_LEN);
+      break;
+    case GRN_ACCESSOR_GET_SUM :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_SUM,
+                   GRN_COLUMN_NAME_SUM_LEN);
+      break;
+    case GRN_ACCESSOR_GET_AVG :
+      GRN_TEXT_PUT(ctx,
+                   buf,
+                   GRN_COLUMN_NAME_AVG,
+                   GRN_COLUMN_NAME_AVG_LEN);
+      break;
+    case GRN_ACCESSOR_GET_COLUMN_VALUE :
+      grn_column_name_(ctx, accessor->obj, buf);
+      show_obj_domain_name = GRN_TRUE;
+      break;
+    case GRN_ACCESSOR_GET_DB_OBJ :
+      grn_text_printf(ctx, buf, "(_db)");
+      break;
+    case GRN_ACCESSOR_LOOKUP :
+      grn_text_printf(ctx, buf, "(_lookup)");
+      break;
+    case GRN_ACCESSOR_FUNCALL :
+      grn_text_printf(ctx, buf, "(_funcall)");
+      break;
+    default :
+      grn_text_printf(ctx, buf, "(unknown:%u)", accessor->action);
+      break;
+    }
+
+    if (show_obj_name || show_obj_domain_name) {
+      grn_obj *target = accessor->obj;
+      char name[GRN_TABLE_MAX_KEY_SIZE];
+      int name_size;
+
+      if (show_obj_domain_name) {
+        target = grn_ctx_at(ctx, target->header.domain);
+      }
+
+      name_size = grn_obj_name(ctx,
+                               target,
+                               name,
+                               GRN_TABLE_MAX_KEY_SIZE);
+      GRN_TEXT_PUTS(ctx, buf, "(");
+      if (name_size == 0) {
+        GRN_TEXT_PUTS(ctx, buf, "anonymous");
+      } else {
+        GRN_TEXT_PUT(ctx, buf, name, name_size);
+      }
+      GRN_TEXT_PUTS(ctx, buf, ")");
+    }
+  }
+  GRN_TEXT_PUTS(ctx, buf, ">");
+
+  return GRN_SUCCESS;
 }
 
 static grn_rc
@@ -504,7 +617,6 @@ grn_column_inspect_common(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
     GRN_TEXT_PUTS(ctx, buf, " range:");
     if (range) {
       grn_inspect_name(ctx, buf, range);
-      grn_obj_unlink(ctx, range);
     } else {
       grn_text_lltoa(ctx, buf, range_id);
     }
@@ -540,6 +652,9 @@ grn_store_inspect_body(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   case GRN_OBJ_COMPRESS_LZ4 :
     GRN_TEXT_PUTS(ctx, buf, "lz4");
     break;
+  case GRN_OBJ_COMPRESS_ZSTD :
+    GRN_TEXT_PUTS(ctx, buf, "zstd");
+    break;
   default:
     break;
   }
@@ -655,7 +770,6 @@ grn_table_key_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   domain = grn_ctx_at(ctx, domain_id);
   if (domain) {
     grn_inspect_name(ctx, buf, domain);
-    grn_obj_unlink(ctx, domain);
   } else if (domain_id) {
     grn_text_lltoa(ctx, buf, domain_id);
   } else {
@@ -681,7 +795,6 @@ grn_table_columns_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
           if (col) {
             if (i++ > 0) { GRN_TEXT_PUTS(ctx, buf, ", "); }
             grn_column_name_(ctx, col, buf);
-            grn_obj_unlink(ctx, col);
           }
         });
     }
@@ -724,7 +837,6 @@ grn_table_default_tokenizer_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
                                        GRN_INFO_DEFAULT_TOKENIZER, NULL);
   if (default_tokenizer) {
     grn_inspect_name(ctx, buf, default_tokenizer);
-    grn_obj_unlink(ctx, default_tokenizer);
   } else {
     GRN_TEXT_PUTS(ctx, buf, "(nil)");
   }
@@ -741,7 +853,6 @@ grn_table_normalizer_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   normalizer = grn_obj_get_info(ctx, obj, GRN_INFO_NORMALIZER, NULL);
   if (normalizer) {
     grn_inspect_name(ctx, buf, normalizer);
-    grn_obj_unlink(ctx, normalizer);
   } else {
     GRN_TEXT_PUTS(ctx, buf, "(nil)");
   }
@@ -753,6 +864,10 @@ static grn_rc
 grn_table_keys_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 {
   grn_table_cursor *tc;
+  int max_n_keys = 10;
+
+  /* TODO */
+  /* max_n_keys = grn_atoi(grn_getenv("GRN_INSPECT_TABLE_MAX_N_KEYS")); */
 
   GRN_TEXT_PUTS(ctx, buf, "keys:[");
   tc = grn_table_cursor_open(ctx, obj, NULL, 0, NULL, 0,
@@ -763,6 +878,10 @@ grn_table_keys_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
     grn_obj key;
     GRN_OBJ_INIT(&key, GRN_BULK, 0, obj->header.domain);
     while ((id = grn_table_cursor_next(ctx, tc))) {
+      if (max_n_keys > 0 && i >= max_n_keys) {
+        GRN_TEXT_PUTS(ctx, buf, ", ...");
+        break;
+      }
       if (i++ > 0) { GRN_TEXT_PUTS(ctx, buf, ", "); }
       grn_table_get_key2(ctx, obj, id, &key);
       grn_inspect(ctx, buf, &key);
@@ -896,6 +1015,22 @@ grn_db_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   return GRN_SUCCESS;
 }
 
+static grn_rc
+grn_time_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
+{
+  int64_t time_raw;
+  int64_t sec;
+  int32_t usec;
+
+  time_raw = GRN_TIME_VALUE(obj);
+  GRN_TIME_UNPACK(time_raw, sec, usec);
+  grn_text_printf(ctx, buffer,
+                  "%" GRN_FMT_INT64D ".%d",
+                  sec, usec);
+
+  return GRN_SUCCESS;
+}
+
 static grn_rc
 grn_geo_point_inspect_point(grn_ctx *ctx, grn_obj *buf, int point)
 {
@@ -1053,7 +1188,6 @@ grn_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
               grn_obj_get_value(ctx, col, id, &value);
               grn_inspect(ctx, buf, &value);
               GRN_OBJ_FIN(ctx, &value);
-              grn_obj_unlink(ctx, col);
             }
           });
       }
@@ -1066,10 +1200,6 @@ grn_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 
   GRN_TEXT_PUTS(ctx, buf, ">");
 
-  if (table) {
-    grn_obj_unlink(ctx, table);
-  }
-
   return GRN_SUCCESS;
 }
 
@@ -1122,6 +1252,9 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
     break;
   case GRN_BULK :
     switch (obj->header.domain) {
+    case GRN_DB_TIME :
+      grn_time_inspect(ctx, buffer, obj);
+      return buffer;
     case GRN_DB_TOKYO_GEO_POINT :
     case GRN_DB_WGS84_GEO_POINT :
       grn_geo_point_inspect(ctx, buffer, obj);
@@ -1136,7 +1269,6 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
       domain = grn_ctx_at(ctx, obj->header.domain);
       if (domain) {
         grn_id type = domain->header.type;
-        grn_obj_unlink(ctx, domain);
         switch (type) {
         case GRN_TABLE_HASH_KEY :
         case GRN_TABLE_PAT_KEY :
@@ -1156,7 +1288,6 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
     domain = grn_ctx_at(ctx, obj->header.domain);
     if (domain) {
       grn_id type = domain->header.type;
-      grn_obj_unlink(ctx, domain);
       switch (type) {
       case GRN_TABLE_HASH_KEY :
       case GRN_TABLE_PAT_KEY :
@@ -1281,7 +1412,7 @@ grn_p(grn_ctx *ctx, grn_obj *obj)
   GRN_TEXT_INIT(&buffer, 0);
   grn_inspect(ctx, &buffer, obj);
   printf("%.*s\n", (int)GRN_TEXT_LEN(&buffer), GRN_TEXT_VALUE(&buffer));
-  grn_obj_unlink(ctx, &buffer);
+  GRN_OBJ_FIN(ctx, &buffer);
 }
 
 void
@@ -1292,7 +1423,7 @@ grn_p_geo_point(grn_ctx *ctx, grn_geo_point *point)
   GRN_WGS84_GEO_POINT_INIT(&obj, 0);
   GRN_GEO_POINT_SET(ctx, &obj, point->latitude, point->longitude);
   grn_p(ctx, &obj);
-  grn_obj_unlink(ctx, &obj);
+  GRN_OBJ_FIN(ctx, &obj);
 }
 
 void
@@ -1303,7 +1434,7 @@ grn_p_ii_values(grn_ctx *ctx, grn_obj *ii)
   GRN_TEXT_INIT(&buffer, 0);
   grn_ii_inspect_values(ctx, (grn_ii *)ii, &buffer);
   printf("%.*s\n", (int)GRN_TEXT_LEN(&buffer), GRN_TEXT_VALUE(&buffer));
-  grn_obj_unlink(ctx, &buffer);
+  GRN_OBJ_FIN(ctx, &buffer);
 }
 
 void
@@ -1314,63 +1445,19 @@ grn_p_expr_code(grn_ctx *ctx, grn_expr_code *code)
   GRN_TEXT_INIT(&buffer, 0);
   grn_expr_code_inspect_indented(ctx, &buffer, code, "");
   printf("%.*s\n", (int)GRN_TEXT_LEN(&buffer), GRN_TEXT_VALUE(&buffer));
-  grn_obj_unlink(ctx, &buffer);
+  GRN_OBJ_FIN(ctx, &buffer);
 }
 
-#ifdef WIN32
-static char *win32_base_dir = NULL;
-const char *
-grn_win32_base_dir(void)
+void
+grn_p_record(grn_ctx *ctx, grn_obj *table, grn_id id)
 {
-  if (!win32_base_dir) {
-    HMODULE dll;
-    const wchar_t *dll_filename = GRN_DLL_FILENAME;
-    wchar_t absolute_dll_filename[MAX_PATH];
-    DWORD absolute_dll_filename_size;
-    dll = GetModuleHandleW(dll_filename);
-    absolute_dll_filename_size = GetModuleFileNameW(dll,
-                                                    absolute_dll_filename,
-                                                    MAX_PATH);
-    if (absolute_dll_filename_size == 0) {
-      win32_base_dir = grn_strdup_raw(".");
-    } else {
-      DWORD ansi_dll_filename_size;
-      ansi_dll_filename_size =
-        WideCharToMultiByte(CP_ACP, 0,
-                            absolute_dll_filename, absolute_dll_filename_size,
-                            NULL, 0, NULL, NULL);
-      if (ansi_dll_filename_size == 0) {
-        win32_base_dir = grn_strdup_raw(".");
-      } else {
-        char *path;
-        win32_base_dir = malloc(ansi_dll_filename_size + 1);
-        WideCharToMultiByte(CP_ACP, 0,
-                            absolute_dll_filename, absolute_dll_filename_size,
-                            win32_base_dir, ansi_dll_filename_size,
-                            NULL, NULL);
-        win32_base_dir[ansi_dll_filename_size] = '\0';
-        if ((path = strrchr(win32_base_dir, '\\'))) {
-          *path = '\0';
-        }
-        path = strrchr(win32_base_dir, '\\');
-        if (path && (grn_strcasecmp(path + 1, "bin") == 0 ||
-                     grn_strcasecmp(path + 1, "lib") == 0)) {
-          *path = '\0';
-        } else {
-          path = win32_base_dir + strlen(win32_base_dir);
-          *path = '\0';
-        }
-        for (path = win32_base_dir; *path; path++) {
-          if (*path == '\\') {
-            *path = '/';
-          }
-        }
-      }
-    }
-  }
-  return win32_base_dir;
+  grn_obj record;
+
+  GRN_RECORD_INIT(&record, 0, grn_obj_id(ctx, table));
+  GRN_RECORD_SET(ctx, &record, id);
+  grn_p(ctx, &record);
+  GRN_OBJ_FIN(ctx, &record);
 }
-#endif
 
 #ifdef WIN32
 int
@@ -1410,6 +1497,13 @@ grn_mkstemp(char *path_template)
 }
 #endif /* WIN32 */
 
+grn_bool
+grn_path_exist(const char *path)
+{
+  struct stat status;
+  return stat(path, &status) == 0;
+}
+
 #ifdef WIN32
 UINT
 grn_windows_encoding_to_code_page(grn_encoding encoding)
