@@ -33,6 +33,8 @@ void data_is_column(void);
 void test_is_column(gconstpointer data);
 void data_is_reference_column(void);
 void test_is_reference_column(gconstpointer data);
+void data_is_index_column(void);
+void test_is_index_column(gconstpointer data);
 void data_is_accessor(void);
 void test_is_accessor(gconstpointer data);
 void data_is_key_accessor(void);
@@ -59,6 +61,8 @@ void data_is_window_function_proc(void);
 void test_is_window_function_proc(gconstpointer data);
 void data_type_to_string(void);
 void test_type_to_string(gconstpointer data);
+void data_name_is_column(void);
+void test_name_is_column(gconstpointer data);
 
 static gchar *tmp_directory;
 static const gchar *database_path;
@@ -257,6 +261,42 @@ test_is_reference_column(gconstpointer data)
   }
 }
 
+void
+data_is_index_column(void)
+{
+#define ADD_DATUM(label, expected, name)                                \
+  gcut_add_datum(label,                                                 \
+                 "expected", G_TYPE_BOOLEAN, expected,                  \
+                 "name", G_TYPE_STRING, name,                           \
+                 NULL)
+
+  ADD_DATUM("table",        FALSE, "Users");
+  ADD_DATUM("value column", FALSE, "Users.age");
+  ADD_DATUM("index column", TRUE,  "Ages.users_age");
+
+#undef ADD_DATUM
+}
+
+void
+test_is_index_column(gconstpointer data)
+{
+  const gchar *name;
+  grn_obj *object;
+
+  assert_send_command("table_create Users TABLE_HASH_KEY ShortText");
+  assert_send_command("column_create Users age COLUMN_SCALAR UInt8");
+  assert_send_command("table_create Ages TABLE_PAT_KEY UInt8");
+  assert_send_command("column_create Ages users_age COLUMN_INDEX Users age");
+
+  name = gcut_data_get_string(data, "name");
+  object = grn_ctx_get(context, name, strlen(name));
+  if (gcut_data_get_string(data, "expected")) {
+    cut_assert_true(grn_obj_is_index_column(context, object));
+  } else {
+    cut_assert_false(grn_obj_is_index_column(context, object));
+  }
+}
+
 void
 data_is_accessor(void)
 {
@@ -715,3 +755,33 @@ test_type_to_string(gconstpointer data)
   cut_assert_equal_string(expected,
                           grn_obj_type_to_string(type));
 }
+
+void
+data_name_is_column(void)
+{
+#define ADD_DATUM(expected, name)                                      \
+  gcut_add_datum(G_STRINGIFY(name),                                    \
+                 "expected", G_TYPE_BOOLEAN, expected,                 \
+                 "name", G_TYPE_STRING, name,                          \
+                 NULL)
+
+  ADD_DATUM(TRUE, "Users.age");
+  ADD_DATUM(FALSE, "Users");
+
+#undef ADD_DATUM
+}
+
+void
+test_name_is_column(gconstpointer data)
+{
+  gboolean expected;
+  const gchar *name;
+
+  expected = gcut_data_get_boolean(data, "expected");
+  name = gcut_data_get_string(data, "name");
+  if (expected) {
+    cut_assert_true(grn_obj_name_is_column(context, name, -1));
+  } else {
+    cut_assert_false(grn_obj_name_is_column(context, name, -1));
+  }
+}
